@@ -209,6 +209,15 @@ class DatabaseManager:
     def get_entry_attributes_from_entry_object(self, entry):
         return entry.custom_properties
 
+    # Return all attachments for an entry uuid
+    def get_entry_attachments_from_entry_uuid(self, uuid):
+        entry = self.db.find_entries(uuid=uuid, first=True)
+        return entry.attachments
+
+    # Return all attachments for an entry object
+    def get_entry_attachments_from_entry_object(self, entry):
+        return entry.attachments
+
     #
     # Entry Checks
     #
@@ -287,6 +296,13 @@ class DatabaseManager:
         else:
             return True
 
+    def has_entry_attachments(self, uuid):
+        entry = self.db.find_entries(uuid=uuid, first=True)
+        if len(entry.attachments) == 0:
+            return False
+        else:
+            return True
+
     #
     # Group Checks
     #
@@ -471,6 +487,22 @@ class DatabaseManager:
         self.changes = True
         self.set_element_mtime(entry)
 
+    def add_entry_attachment(self, uuid, bytes, filename):
+        entry = self.db.find_entries(uuid=uuid, first=True)
+        attachment_id = self.db.add_binary(bytes)
+        attachment = entry.add_attachment(attachment_id, filename)
+        self.changes = True
+        return attachment
+
+    def delete_entry_attachment(self, uuid, attachment):
+        entry = self.db.find_entries(uuid=uuid, first=True)
+        try:
+            self.db.delete_binary(attachment.id)
+            entry.delete_attachment(attachment)
+        except Exception:
+            self.logging_manager.warning("Skipping attachment handling...")
+        self.changes = True
+
     # Move an entry to another group
     def move_entry(self, uuid, destination_group_object):
         entry = self.db.find_entries(uuid=uuid, first=True)
@@ -501,11 +533,13 @@ class DatabaseManager:
     def set_group_notes(self, uuid, notes):
         group = self.db.find_groups(uuid=uuid, first=True)
         group.notes = notes
+        self.changes = True
         self.set_element_mtime(group)
 
     def set_group_icon(self, uuid, icon):
         group = self.db.find_groups(uuid=uuid, first=True)
         group.icon = icon
+        self.changes = True
         self.set_element_mtime(group)
 
     # Move an group
@@ -560,26 +594,73 @@ class DatabaseManager:
     def search(self, string, fulltext, global_search=True, path=None):
         uuid_list = []
 
+        # if fulltext is False:
+        #     for group in self.db.find_groups(name="(?i)" + string.replace(" ", ".*"), recursive=global_search, path=path, regex=True):
+        #         if group.is_root_group is False:
+        #             uuid_list.append(group.uuid)
+        # else:
+        #     for group in self.db.groups:
+        #         if group.is_root_group is False and group.uuid not in uuid_list:
+        #             if string.lower() in self.get_group_notes_from_group_object(group):
+        #                 uuid_list.append(group.uuid)
+
+        # if fulltext is False:
+        #     for entry in self.db.find_entries(title="(?i)" + string.replace(" ", ".*"), recursive=global_search, path=path, regex=True):
+        #         uuid_list.append(entry.uuid)
+        # else:
+        #     for entry in self.db.entries:
+        #         if entry.uuid not in uuid_list:
+        #             if string.lower() in self.get_entry_username_from_entry_object(entry):
+        #                 uuid_list.append(entry.uuid)
+        #             elif string.lower() in self.get_entry_notes_from_entry_object(entry):
+        #                 uuid_list.append(entry.uuid)
+
+        # Workaround
         if fulltext is False:
-            for group in self.db.find_groups(name="(?i)" + string.replace(" ", ".*"), recursive=global_search, path=path, regex=True):
-                if group.is_root_group is False:
-                    uuid_list.append(group.uuid)
+            for group in self.db.groups:
+                if group.is_root_group is False and group.uuid not in uuid_list:
+                    if group.name is not None:
+                        if string.lower() in group.name.lower():
+                            if global_search is True:
+                                uuid_list.append(group.uuid)
+                            else:
+                                if group.path[:-1].rsplit("/", 1)[0] == path.replace("//", ""):
+                                    uuid_list.append(group.uuid)
         else:
             for group in self.db.groups:
                 if group.is_root_group is False and group.uuid not in uuid_list:
-                    if string.lower() in self.get_group_notes_from_group_object(group):
-                        uuid_list.append(group.uuid)
+                    if string.lower() in self.get_group_notes_from_group_object(group).lower():
+                        if global_search is True:
+                            uuid_list.append(group.uuid)
+                        else:
+                            if group.path[:-1].rsplit("/", 1)[0] == path.replace("//", ""):
+                                uuid_list.append(group.uuid)
 
         if fulltext is False:
-            for entry in self.db.find_entries(title="(?i)" + string.replace(" ", ".*"), recursive=global_search, path=path, regex=True):
-                uuid_list.append(entry.uuid)
+            for entry in self.db.entries:
+                if entry.uuid not in uuid_list:
+                    if entry.title is not None:
+                        if string.lower() in entry.title.lower():
+                            if global_search is True:
+                                uuid_list.append(entry.uuid)
+                            else:
+                                if entry.path.rsplit("/", 1)[0] == path.replace("//", ""):
+                                    uuid_list.append(entry.uuid)
         else:
             for entry in self.db.entries:
                 if entry.uuid not in uuid_list:
-                    if string.lower() in self.get_entry_username_from_entry_object(entry):
-                        uuid_list.append(entry.uuid)
-                    elif string.lower() in self.get_entry_notes_from_entry_object(entry):
-                        uuid_list.append(entry.uuid)
+                    if string.lower() in self.get_entry_username_from_entry_object(entry).lower():
+                        if global_search is True:
+                            uuid_list.append(entry.uuid)
+                        else:
+                            if entry.path.rsplit("/", 1)[0] == path.replace("//", ""):
+                                uuid_list.append(entry.uuid)
+                    elif string.lower() in self.get_entry_notes_from_entry_object(entry).lower():
+                        if global_search is True:
+                            uuid_list.append(entry.uuid)
+                        else:
+                            if entry.path.rsplit("/", 1)[0] == path.replace("//", ""):
+                                uuid_list.append(entry.uuid)
 
         return uuid_list
 
@@ -593,6 +674,9 @@ class DatabaseManager:
     def check_is_group_object(self, group):
         return hasattr(group, "name")
 
+    def get_attachment_from_id(self, attachment_id):
+        return self.db.find_attachments(id=attachment_id, first=True)
+
     #
     # Properties
     #
