@@ -1,9 +1,13 @@
-from gi.repository import Gtk
+from gi.repository import Gtk, Gio, GLib
 from gettext import gettext as _
 from passwordsafe.notes_dialog import NotesDialog
+from passwordsafe.history_buffer import HistoryEntryBuffer, HistoryTextBuffer
 import passwordsafe.passphrase_generator
 import passwordsafe.password_generator
 import passwordsafe.config_manager
+import threading
+import subprocess, os
+import uuid as u
 
 
 class EntryPage:
@@ -55,12 +59,13 @@ class EntryPage:
         builder.add_from_resource("/org/gnome/PasswordSafe/entry_page.ui")
 
         entry_uuid = self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group)
-        scrolled_page = self.unlocked_database.stack.get_child_by_name(entry_uuid)
+        scrolled_page = self.unlocked_database.stack.get_child_by_name(entry_uuid.urn)
 
         if self.unlocked_database.database_manager.has_entry_name(entry_uuid) is True or add_all is True:
             if scrolled_page.name_property_row is NotImplemented:
                 scrolled_page.name_property_row = builder.get_object("name_property_row")
                 scrolled_page.name_property_value_entry = builder.get_object("name_property_value_entry")
+                scrolled_page.name_property_value_entry.set_buffer(HistoryEntryBuffer([]))
                 value = self.unlocked_database.database_manager.get_entry_name_from_entry_uuid(entry_uuid)
                 if self.unlocked_database.database_manager.has_entry_name(entry_uuid) is True:
                     scrolled_page.name_property_value_entry.set_text(value)
@@ -84,6 +89,7 @@ class EntryPage:
             if scrolled_page.username_property_row is NotImplemented:
                 scrolled_page.username_property_row = builder.get_object("username_property_row")
                 scrolled_page.username_property_value_entry = builder.get_object("username_property_value_entry")
+                scrolled_page.username_property_value_entry.set_buffer(HistoryEntryBuffer([]))
                 value = self.unlocked_database.database_manager.get_entry_username_from_entry_uuid(entry_uuid)
                 if self.unlocked_database.database_manager.has_entry_username(entry_uuid) is True:
                     scrolled_page.username_property_value_entry.set_text(value)
@@ -110,6 +116,7 @@ class EntryPage:
                 scrolled_page.password_property_value_entry = builder.get_object("password_property_value_entry")
                 scrolled_page.show_password_button = builder.get_object("show_password_button")
                 scrolled_page.generate_password_button = builder.get_object("generate_password_button")
+                scrolled_page.password_property_value_entry.set_buffer(HistoryEntryBuffer([]))
                 value = self.unlocked_database.database_manager.get_entry_password_from_entry_uuid(entry_uuid)
 
                 if self.unlocked_database.database_manager.has_entry_password(entry_uuid) is True:
@@ -150,6 +157,7 @@ class EntryPage:
             if scrolled_page.url_property_row is NotImplemented:
                 scrolled_page.url_property_row = builder.get_object("url_property_row")
                 scrolled_page.url_property_value_entry = builder.get_object("url_property_value_entry")
+                scrolled_page.url_property_value_entry.set_buffer(HistoryEntryBuffer([]))
                 value = self.unlocked_database.database_manager.get_entry_url_from_entry_uuid(entry_uuid)
                 if self.unlocked_database.database_manager.has_entry_url(entry_uuid) is True:
                     scrolled_page.url_property_value_entry.set_text(value)
@@ -177,6 +185,7 @@ class EntryPage:
 
                 builder.get_object("notes_detach_button").connect("clicked", self.on_notes_detach_button_clicked)
 
+                scrolled_page.notes_property_value_entry.set_buffer(HistoryTextBuffer([]))
                 buffer = scrolled_page.notes_property_value_entry.get_buffer()
                 value = self.unlocked_database.database_manager.get_entry_notes_from_entry_uuid(entry_uuid)
                 if self.unlocked_database.database_manager.has_entry_notes(entry_uuid) is True:
@@ -305,6 +314,18 @@ class EntryPage:
             elif scrolled_page.icon_property_row is not NotImplemented:
                 properties_list_box.add(scrolled_page.icon_property_row)
 
+        if scrolled_page.attachment_property_row is NotImplemented:
+            scrolled_page.attachment_property_row = builder.get_object("attachment_property_row")
+            scrolled_page.attachment_list_box = builder.get_object("attachment_list_box")
+            for attachment in self.unlocked_database.database_manager.get_entry_attachments_from_entry_uuid(entry_uuid):
+                self.add_attachment_row(attachment)
+
+            scrolled_page.attachment_list_box.add(builder.get_object("add_attachment_row"))
+            scrolled_page.attachment_list_box.connect("row-activated", self.on_attachment_list_box_activated)
+            properties_list_box.add(scrolled_page.attachment_property_row)
+        elif scrolled_page.attachment_property_row is not NotImplemented:
+            properties_list_box.add(scrolled_page.attachment_property_row)
+
         if scrolled_page.attributes_property_row is NotImplemented:
             scrolled_page.attributes_property_row = builder.get_object("attributes_property_row")
             scrolled_page.attributes_key_entry = builder.get_object("attributes_key_entry")
@@ -334,7 +355,7 @@ class EntryPage:
             properties_list_box.add(scrolled_page.show_all_row)
 
     def add_attribute_property_row(self, key, value):
-        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group))
+        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group).urn)
 
         builder = Gtk.Builder()
         builder.add_from_resource("/org/gnome/PasswordSafe/entry_page.ui")
@@ -345,6 +366,7 @@ class EntryPage:
         attribute_property_name_label = builder.get_object("attribute_property_name_label")
         attribute_key_edit_button = builder.get_object("attribute_key_edit_button")
         attribute_value_entry = builder.get_object("attribute_value_entry")
+        attribute_value_entry.set_buffer(HistoryEntryBuffer([]))
         attribute_remove_button = builder.get_object("attribute_remove_button")
 
         attribute_property_row.set_name(key)
@@ -366,7 +388,7 @@ class EntryPage:
     def on_show_all_properties_button_clicked(self, widget):
         self.unlocked_database.start_database_lock_timer()
         entry_uuid = self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group)
-        scrolled_page = self.unlocked_database.stack.get_child_by_name(entry_uuid)
+        scrolled_page = self.unlocked_database.stack.get_child_by_name(entry_uuid.urn)
 
         for row in scrolled_page.properties_list_box.get_children():
             scrolled_page.properties_list_box.remove(row)
@@ -377,7 +399,7 @@ class EntryPage:
         self.unlocked_database.start_database_lock_timer()
         entry_uuid = self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group)
 
-        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group))
+        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group).urn)
         scrolled_page.set_made_database_changes(True)
 
         if type == "name":
@@ -406,7 +428,7 @@ class EntryPage:
 
         self.unlocked_database.start_database_lock_timer()
         entry_uuid = self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group)
-        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group))
+        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group).urn)
 
         if str(self.unlocked_database.database_manager.get_entry_icon_from_entry_uuid(entry_uuid)) == button.get_name():
             return
@@ -420,7 +442,7 @@ class EntryPage:
 
         self.unlocked_database.start_database_lock_timer()
         entry_uuid = self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group)
-        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group))
+        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group).urn)
 
         if button.get_name() != "NoneColorButton":
             image = button.get_children()[0]
@@ -477,7 +499,7 @@ class EntryPage:
 
     def on_attributes_add_button_clicked(self, widget):
         entry_uuid = self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group)
-        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group))
+        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group).urn)
 
         key = scrolled_page.attributes_key_entry.get_text()
         value = scrolled_page.attributes_value_entry.get_text()
@@ -502,7 +524,7 @@ class EntryPage:
 
     def on_attribute_remove_button_clicked(self, button):
         entry_uuid = self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group)
-        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group))
+        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group).urn)
 
         parent = button.get_parent().get_parent().get_parent()
         key = parent.get_name()
@@ -565,10 +587,121 @@ class EntryPage:
         attribute_entry_box.add(button)
         attribute_entry_box.reorder_child(button, 0)
 
+    #
+    # Attachment Handling
+    #
+
+    def on_attachment_list_box_activated(self, widget, list_box_row):
+        if list_box_row.get_name() == "AddAttachmentRow":
+            self.on_add_attachment_row_clicked()
+        else:
+            self.on_attachment_row_clicked(self.unlocked_database.database_manager.get_attachment_from_id(list_box_row.get_name()))
+
+    def on_add_attachment_row_clicked(self):
+        self.unlocked_database.start_database_lock_timer()
+        select_dialog = Gtk.FileChooserNative.new(
+            # NOTE: Filechooser title for selecting attachment file
+            _("Select attachment"), self.unlocked_database.window, Gtk.FileChooserAction.OPEN,
+            _("Add"), None)
+        select_dialog.set_modal(True)
+        select_dialog.set_local_only(False)
+        select_dialog.set_select_multiple(True)
+
+        response = select_dialog.run()
+
+        if response == Gtk.ResponseType.ACCEPT:
+            entry_uuid = self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group)
+            for uri in select_dialog.get_uris():
+                attachment = Gio.File.new_for_uri(uri)
+                bytes = attachment.load_bytes()[0].get_data()
+                filename = attachment.get_basename()
+                self.add_attachment_row(self.unlocked_database.database_manager.add_entry_attachment(entry_uuid, bytes, filename))
+
+    def add_attachment_row(self, attachment):
+        entry_uuid = self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group)
+        scrolled_page = self.unlocked_database.stack.get_child_by_name(entry_uuid.urn)
+
+        builder = Gtk.Builder()
+        builder.add_from_resource("/org/gnome/PasswordSafe/entry_page.ui")
+
+        attachment_row = builder.get_object("attachment_row")
+        attachment_row.set_name(str(attachment.id))
+        builder.get_object("attachment_label").set_label(attachment.filename)
+        builder.get_object("attachment_download_button").connect("clicked", self.on_attachment_download_button_clicked, attachment)
+        builder.get_object("attachment_delete_button").connect("clicked", self.on_attachment_delete_button_clicked, entry_uuid, attachment, attachment_row)
+
+        scrolled_page.attachment_list_box.insert(attachment_row, len(scrolled_page.attachment_list_box)-1)
+
+    def on_attachment_row_clicked(self, attachment):
+        builder = Gtk.Builder()
+        builder.add_from_resource("/org/gnome/PasswordSafe/attachment_warning_dialog.ui")
+        warning_dialog = builder.get_object("warning_dialog")
+        warning_dialog.set_destroy_with_parent(True)
+        warning_dialog.set_modal(True)
+        warning_dialog.set_transient_for(self.unlocked_database.window)
+
+        builder.get_object("back_button").connect("clicked", self.on_warning_dialog_back_button_clicked, warning_dialog)
+        builder.get_object("proceed_button").connect("clicked", self.on_warning_dialog_proceed_button_clicked, warning_dialog, attachment)
+
+        warning_dialog.present()
+
+    def on_attachment_download_button_clicked(self, button, attachment):
+        save_dialog = Gtk.FileChooserNative.new(
+            # NOTE: Filechooser title for downloading an attachment
+            _("Save attachment"), self.unlocked_database.window, Gtk.FileChooserAction.SAVE,
+            _("Save"), None)
+        save_dialog.set_do_overwrite_confirmation(True)
+        save_dialog.set_current_name(attachment.filename)
+        save_dialog.set_modal(True)
+        save_dialog.set_local_only(False)
+
+        response = save_dialog.run()
+
+        if response == Gtk.ResponseType.ACCEPT:
+            bytes = self.unlocked_database.database_manager.db.binaries[attachment.id]
+            self.save_to_disk(save_dialog.get_filename(), bytes)
+
+    def on_attachment_delete_button_clicked(self, button, entry_uuid, attachment, attachment_row):
+        self.unlocked_database.database_manager.delete_entry_attachment(entry_uuid, attachment)
+        attachment_row.destroy()
+
+        builder = Gtk.Builder()
+        builder.add_from_resource("/org/gnome/PasswordSafe/entry_page.ui")
+
+        entry_uuid = self.unlocked_database.database_manager.get_entry_uuid_from_entry_object(self.unlocked_database.current_group)
+        scrolled_page = self.unlocked_database.stack.get_child_by_name(entry_uuid.urn)
+
+        for row in scrolled_page.attachment_list_box.get_children():
+            if row.get_name() != "AddAttachmentRow":
+                row.destroy()
+
+        for attachment in self.unlocked_database.database_manager.get_entry_attachments_from_entry_uuid(entry_uuid):
+            self.add_attachment_row(attachment)
+
     #
     # Helper
     #
 
+    def save_to_disk(self, filepath, bytes):
+        file = Gio.File.new_for_path(filepath)
+        stream = Gio.File.create(file, Gio.FileCreateFlags.PRIVATE, None)
+        file_buffer = Gio.OutputStream.write_bytes(stream, GLib.Bytes.new(bytes), None)
+        stream.close()
+
+    def open_tmp_file(self, bytes, filename):
+        (file, stream) = Gio.File.new_tmp(filename + ".XXXXXX")
+        stream.get_output_stream().write_bytes(GLib.Bytes.new(bytes))
+        stream.close()
+        self.unlocked_database.scheduled_tmpfiles_deletion.append(file)
+        subprocess.call("xdg-open " + file.get_path(), shell=True)
+
+    def on_warning_dialog_back_button_clicked(self, button, dialog):
+        dialog.close()
+
+    def on_warning_dialog_proceed_button_clicked(self, button, dialog, attachment):
+        dialog.close()
+        self.open_tmp_file(self.unlocked_database.database_manager.db.binaries[attachment.id], attachment.filename)
+
     def change_password_entry_visibility(self, entry, toggle_button):
         toggle_button.connect("toggled", self.on_show_password_button_toggled, entry)
 
@@ -583,7 +716,7 @@ class EntryPage:
 
         if scrolled_page.password_property_value_entry.get_text().startswith("{REF:P"):
             try:
-                password = self.unlocked_database.database_manager.get_entry_password_from_entry_uuid(self.unlocked_database.hex_to_base64(self.unlocked_database.reference_to_hex_uuid(scrolled_page.password_property_value_entry.get_text())))
+                password = self.unlocked_database.database_manager.get_entry_password_from_entry_uuid(u.UUID(self.unlocked_database.reference_to_hex_uuid(scrolled_page.password_property_value_entry.get_text())))
             except(Exception):
                 password = scrolled_page.password_property_value_entry.get_text()
         else:
@@ -591,4 +724,3 @@ class EntryPage:
 
         if password is not NotImplemented:
             scrolled_page.password_level_bar.set_value(float(passwordsafe.password_generator.strength(password)))
-
