@@ -4,6 +4,7 @@ from gi.repository.GdkPixbuf import Pixbuf
 from passwordsafe.database_manager import DatabaseManager
 from passwordsafe.create_database import CreateDatabase
 from passwordsafe.container_page import ContainerPage
+from passwordsafe.error_info_bar import ErrorInfoBar
 from passwordsafe.unlock_database import UnlockDatabase
 import passwordsafe.config_manager
 import os
@@ -32,6 +33,9 @@ class MainWindow(Gtk.ApplicationWindow):
         super().__init__(*args, **kwargs)
         self.logging_manager = kwargs['application'].get_logger()
 
+        self._info_bar = None
+        self._info_bar_response_id = None
+
         self.assemble_window()
 
         if Gio.Application.get_default().development_mode is True:
@@ -137,7 +141,7 @@ class MainWindow(Gtk.ApplicationWindow):
                 group_page = True
             else:
                 group_page = False
-            scrolled_page = db.stack.get_child_by_name(page_uuid)
+            scrolled_page = db.stack.get_child_by_name(page_uuid.urn)
 
             # For Group/Entry Edit Page
             if scrolled_page.edit_page is True and group_page is True:
@@ -240,6 +244,7 @@ class MainWindow(Gtk.ApplicationWindow):
                 # Responsive Container
                 hdy = Handy.Column()
                 hdy.set_maximum_width(400)
+                hdy.props.vexpand = True
                 hdy.add(builder.get_object("select_box"))
 
                 self.first_start_grid.add(hdy)
@@ -277,40 +282,82 @@ class MainWindow(Gtk.ApplicationWindow):
     # Open Database Methods
     #
 
+    def _on_info_bar_response(self, info_bar=None, response_id=None):
+        self._info_bar.hide()
+        self._info_bar.disconnect(self._info_bar_response_id)
+        self._info_bar = None
+        self._info_bar_response_id = None
+
     def open_filechooser(self, widget, none):
+        if self._info_bar is not None:
+            self._on_info_bar_response()
+
         filechooser_opening_dialog = Gtk.FileChooserNative.new(
             # NOTE: Filechooser title for opening an existing keepass safe kdbx file
             _("Choose a Keepass safe"), self, Gtk.FileChooserAction.OPEN,
             None, None)
 
+        supported_mime_types = [
+            "application/x-keepass2",
+            "application/octet-stream"
+        ]
+
         filter_text = Gtk.FileFilter()
         # NOTE: KeePass + version number is a proper name, do not translate
         filter_text.set_name(_("KeePass 3.1/4 Database"))
-        filter_text.add_mime_type("application/x-keepass2")
-        filter_text.add_mime_type("application/octet-stream")
+        for mime_type in supported_mime_types:
+            filter_text.add_mime_type(mime_type)
         filechooser_opening_dialog.add_filter(filter_text)
         filechooser_opening_dialog.set_local_only(False)
 
         response = filechooser_opening_dialog.run()
 
         if response == Gtk.ResponseType.ACCEPT:
-            self.logging_manager.debug(
-                "File selected: " + filechooser_opening_dialog.get_filename())
+            db_filename = filechooser_opening_dialog.get_filename()
+            self.logging_manager.debug("File selected: " + db_filename)
+
+            db_gfile = Gio.File.new_for_path(db_filename)
+            try:
+                db_file_info = db_gfile.query_info(
+                    "standard::content-type", Gio.FileQueryInfoFlags.NONE,
+                    None)
+            except GLib.Error as e:
+                self.logging_manager.debug(
+                    "Unable to query info for file {}: {}".format(
+                        db_filename, e.message))
+                return
+
+            file_content_type = db_file_info.get_content_type()
+            file_mime_type = Gio.content_type_get_mime_type(file_content_type)
+            if file_mime_type not in supported_mime_types:
+                self.logging_manager.debug(
+                    "Unsupported mime type {}".format(file_mime_type))
+                main_message = _(
+                    'Unable to open file "{}".'.format(db_filename))
+                mime_desc = Gio.content_type_get_description(file_content_type)
+                second_message = _(
+                    "File type {} ({}) is not supported.".format(
+                        mime_desc, file_mime_type))
+                self._info_bar = ErrorInfoBar(main_message, second_message)
+                self._info_bar_response_id = self._info_bar.connect(
+                    "response", self._on_info_bar_response)
+                first_child = self.first_start_grid.get_children()[0]
+                self.first_start_grid.attach_next_to(
+                    self._info_bar, first_child, Gtk.PositionType.TOP, 1, 1)
+                return
 
             database_already_opened = False
 
             for db in self.opened_databases:
-                if db.database_manager.database_path == filechooser_opening_dialog.get_filename():
+                if db.database_manager.database_path == db_filename:
                     database_already_opened = True
                     page_num = self.container.page_num(db.parent_widget)
                     self.container.set_current_page(page_num)
                     db.show_database_action_revealer("Database already opened")
 
             if database_already_opened is False:
-                tab_title = self.create_tab_title_from_filepath(
-                    filechooser_opening_dialog.get_filename())
-                self.start_database_opening_routine(
-                    tab_title, filechooser_opening_dialog.get_filename())
+                tab_title = self.create_tab_title_from_filepath(db_filename)
+                self.start_database_opening_routine(tab_title, db_filename)
         elif response == Gtk.ResponseType.CANCEL:
             self.logging_manager.debug("File selection canceled")
 
@@ -461,6 +508,7 @@ class MainWindow(Gtk.ApplicationWindow):
             if db.window.container.page_num(db.parent_widget) == page_num:
                 db.database_locked = True
                 db.stop_save_loop()
+                db.clipboard.clear()
                 is_contained = True
                 if db.database_manager.made_database_changes() is True:
                     if passwordsafe.config_manager.get_save_automatically() is True:
@@ -509,6 +557,7 @@ class MainWindow(Gtk.ApplicationWindow):
         for db in self.opened_databases:
             db.cancel_timers()
             db.unregister_dbus_signal()
+            db.clipboard.clear()
 
         if len(self.databases_to_save) > 0:
             save_thread = threading.Thread(target=self.threaded_database_saving)
@@ -568,6 +617,12 @@ class MainWindow(Gtk.ApplicationWindow):
                 unsaved_database_row.show_all()
                 unsaved_databases_list_box.add(unsaved_database_row)
 
+                for tmpfile in db.scheduled_tmpfiles_deletion:
+                    try:
+                        tmpfile.delete()
+                    except Exception:
+                        self.logging_manager.warning("Skipping deletion of tmpfile...")
+
             self.quit_dialog.present()
 
             return(True)
@@ -576,12 +631,14 @@ class MainWindow(Gtk.ApplicationWindow):
                 db.cancel_timers()
                 db.unregister_dbus_signal()
                 db.show_save_dialog(True, False, True)
+                db.clipboard.clear()
                 return(True)
         else:
             self.save_window_size()
 
             for db in self.opened_databases:
                 db.cancel_timers()
+                db.clipboard.clear()
 
     #
     # Gio Actions
@@ -687,6 +744,10 @@ class MainWindow(Gtk.ApplicationWindow):
             action_db.on_sort_menu_button_entry_clicked(action, param, arg)
         elif name == "on_selection_popover_button_clicked":
             action_db.selection_ui.on_selection_popover_button_clicked(action, param, arg)
+        elif name == "on_edit_undo":
+            action_db.undo_redo_receiver(arg)
+        elif name == "on_edit_redo":
+            action_db.undo_redo_receiver(arg)
 
     # Add Global Accelerator Actions
     def add_global_accelerator_actions(self):
@@ -706,6 +767,14 @@ class MainWindow(Gtk.ApplicationWindow):
         add_group_action.connect("activate", self.execute_accel_action, "add_action", "group")
         self.application.add_action(add_group_action)
 
+        undo_action = Gio.SimpleAction.new("undo")
+        undo_action.connect("activate", self.execute_gio_action, "on_edit_undo", "undo")
+        self.application.add_action(undo_action)
+
+        redo_action = Gio.SimpleAction.new("redo")
+        redo_action.connect("activate", self.execute_gio_action, "on_edit_redo", "redo")
+        self.application.add_action(redo_action)
+
     # Accelerator Action Handler
     def execute_accel_action(self, action, param, name, arg=None):
         action_db = self.find_action_db()
@@ -724,7 +793,7 @@ class MainWindow(Gtk.ApplicationWindow):
                 return
 
             group_uuid = action_db.database_manager.get_group_uuid_from_group_object(action_db.current_group)
-            scrolled_page = action_db.stack.get_child_by_name(group_uuid)
+            scrolled_page = action_db.stack.get_child_by_name(group_uuid.urn)
             if scrolled_page.edit_page is True:
                 return
 
@@ -750,4 +819,3 @@ class MainWindow(Gtk.ApplicationWindow):
         self.quit_dialog.destroy()
         self.save_window_size()
         self.application.quit()
-            
