@@ -20,7 +20,7 @@ class ResponsiveUI:
     #
 
     def action_bar(self):
-        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_group_uuid_from_group_object(self.unlocked_database.current_group))
+        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_group_uuid_from_group_object(self.unlocked_database.current_group).urn)
         if self.unlocked_database.window.mobile_width is True:
             if len(self.unlocked_database.pathbar.viewport.get_children()) is not 0:
                 self.unlocked_database.pathbar.viewport.remove(self.unlocked_database.pathbar)
@@ -44,7 +44,7 @@ class ResponsiveUI:
                 self.unlocked_database.pathbar.pathbar_box.show()
 
     def headerbar_title(self):
-        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_group_uuid_from_group_object(self.unlocked_database.current_group))
+        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_group_uuid_from_group_object(self.unlocked_database.current_group).urn)
         if self.unlocked_database.window.mobile_width is True and self.unlocked_database.selection_ui.selection_mode_active is False:
             if len(self.unlocked_database.builder.get_object("title_box").get_children()) is not 0:
                 return
@@ -72,7 +72,7 @@ class ResponsiveUI:
             self.unlocked_database.builder.get_object("pathbar_button_back_revealer").set_reveal_child(False)
 
     def headerbar_selection_button(self):
-        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_group_uuid_from_group_object(self.unlocked_database.current_group))
+        scrolled_page = self.unlocked_database.stack.get_child_by_name(self.unlocked_database.database_manager.get_group_uuid_from_group_object(self.unlocked_database.current_group).urn)
         if self.unlocked_database.selection_ui.selection_mode_active is True:
             return
 
@@ -86,4 +86,3 @@ class ResponsiveUI:
                 return
 
             self.unlocked_database.builder.get_object("selection_button_revealer").set_reveal_child(True)
-
