@@ -1,6 +1,7 @@
 from gi.repository import Gio, Gtk, GLib, Handy, Pango
 from passwordsafe.database_manager import DatabaseManager
 from passwordsafe.unlocked_database import UnlockedDatabase
+from pykeepass.exceptions import CredentialsIntegrityError
 import passwordsafe.config_manager
 import ntpath
 import os
@@ -263,7 +264,7 @@ class UnlockDatabase:
         try:
             self.database_manager = DatabaseManager(self.database_filepath, password=self.password_only, keyfile=None, logging_manager=self.window.logging_manager)
             GLib.idle_add(self.password_unlock_success)
-        except(OSError, ValueError, AttributeError, core.ChecksumError):
+        except(OSError, ValueError, AttributeError, core.ChecksumError, CredentialsIntegrityError):
             GLib.idle_add(self.password_unlock_failure)
 
     def password_unlock_success(self):
@@ -375,7 +376,7 @@ class UnlockDatabase:
         try:
             self.database_manager = DatabaseManager(self.database_filepath, password=None, keyfile=self.keyfile_path, logging_manager=self.window.logging_manager)
             GLib.idle_add(self.keyfile_unlock_success)
-        except(OSError, IndexError, ValueError, AttributeError, core.ChecksumError):
+        except(OSError, IndexError, ValueError, AttributeError, core.ChecksumError, CredentialsIntegrityError):
             GLib.idle_add(self.keyfile_unlock_failure)
 
     def keyfile_unlock_success(self):
@@ -507,7 +508,7 @@ class UnlockDatabase:
         try:
             self.database_manager = DatabaseManager(self.database_filepath, password=self.password_composite, keyfile=self.composite_keyfile_path, logging_manager=self.window.logging_manager)
             GLib.idle_add(self.composite_unlock_success)
-        except(OSError, IndexError, ValueError, AttributeError, core.ChecksumError):
+        except(OSError, IndexError, ValueError, AttributeError, core.ChecksumError, CredentialsIntegrityError):
             GLib.idle_add(self.composite_unlock_failure)
 
     def composite_unlock_success(self):
@@ -580,7 +581,10 @@ class UnlockDatabase:
 
             opened = Gio.File.new_for_path(self.database_filepath)
             backup = Gio.File.new_for_path(cache_dir + "/" + os.path.splitext(ntpath.basename(self.database_filepath))[0] + "_backup_" + datetime.datetime.fromtimestamp(time.time()).strftime('%Y-%m-%d_%H:%M:%S') + ".kdbx")
-            opened.copy(backup, Gio.FileCopyFlags.NONE)
+            try:
+                opened.copy(backup, Gio.FileCopyFlags.NONE)
+            except GLib.Error:
+                self.window.logging_manager.warning("Could not copy database file to backup location. This most likely happened because the database is located on a network drive, and Password Safe doesn't have network permission. Either disable development-backup-mode or if PasswordSafe runs as Flatpak grant network permission")
 
         already_added = False
         list = []
