@@ -21,6 +21,7 @@ import passwordsafe.config_manager
 import re
 import time
 import threading
+import uuid as u
 
 
 class UnlockedDatabase:
@@ -57,6 +58,7 @@ class UnlockedDatabase:
     current_group = NotImplemented
     accelerators = NotImplemented
     scheduled_page_destroy = []
+    scheduled_tmpfiles_deletion = []
     clipboard = NotImplemented
     list_box_sorting = NotImplemented
     clipboard_timer = NotImplemented
@@ -231,15 +233,15 @@ class UnlockedDatabase:
             scrolled_window.show_all()
 
             stack_page_uuid = self.database_manager.get_group_uuid_from_group_object(self.current_group)
-            if self.stack.get_child_by_name(stack_page_uuid) is not None:
-                stack_page = self.stack.get_child_by_name(stack_page_uuid)
+            if self.stack.get_child_by_name(stack_page_uuid.urn) is not None:
+                stack_page = self.stack.get_child_by_name(stack_page_uuid.urn)
                 stack_page.destroy()
 
             self.add_stack_page(scrolled_window)
             self.group_page.insert_group_properties_into_listbox(scrolled_window.properties_list_box)
             self.group_page.set_group_edit_page_headerbar()
         # If the stack page with current group's uuid isn't existing - we need to create it (first time opening of group/entry)
-        elif self.stack.get_child_by_name(self.database_manager.get_group_uuid_from_group_object(self.current_group)) is None and self.stack.get_child_by_name(self.database_manager.get_entry_uuid_from_entry_object(self.current_group)) is None and edit_group is False:
+        elif self.stack.get_child_by_name(self.database_manager.get_group_uuid_from_group_object(self.current_group).urn) is None and self.stack.get_child_by_name(self.database_manager.get_entry_uuid_from_entry_object(self.current_group).urn) is None and edit_group is False:
             self.database_manager.set_element_atime(self.current_group)
             # Create not existing stack page for group
             if self.database_manager.check_is_group(self.database_manager.get_group_uuid_from_group_object(self.current_group)) is True:
@@ -255,11 +257,11 @@ class UnlockedDatabase:
                 # Responsive Container
                 list_box.set_name("BrowserListBox")
                 list_box.set_valign(Gtk.Align.START)
+                list_box.set_margin_top(18)
+                list_box.set_margin_bottom(18)
 
                 hdy_browser = Handy.Column()
                 hdy_browser.set_maximum_width(700)
-                hdy_browser.set_margin_top(18)
-                hdy_browser.set_margin_bottom(18)
                 hdy_browser.add(list_box)
                 overlay.add(hdy_browser)
 
@@ -305,18 +307,18 @@ class UnlockedDatabase:
             self.database_manager.set_element_atime(self.current_group)
             # For group
             if self.database_manager.check_is_group(self.database_manager.get_group_uuid_from_group_object(self.current_group)) is True:
-                self.stack.set_visible_child_name(self.database_manager.get_group_uuid_from_group_object(self.current_group))
+                self.stack.set_visible_child_name(self.database_manager.get_group_uuid_from_group_object(self.current_group).urn)
                 self.set_browser_headerbar()
             # For entry
             else:
-                self.stack.set_visible_child_name(self.database_manager.get_entry_uuid_from_entry_object(self.current_group))
+                self.stack.set_visible_child_name(self.database_manager.get_entry_uuid_from_entry_object(self.current_group).urn)
                 self.entry_page.set_entry_page_headerbar()
 
     def add_stack_page(self, scrolled_window):
         if self.database_manager.check_is_group(self.database_manager.get_group_uuid_from_group_object(self.current_group)) is True:
-            self.stack.add_named(scrolled_window, self.database_manager.get_group_uuid_from_group_object(self.current_group))
+            self.stack.add_named(scrolled_window, self.database_manager.get_group_uuid_from_group_object(self.current_group).urn)
         else:
-            self.stack.add_named(scrolled_window, self.database_manager.get_entry_uuid_from_entry_object(self.current_group))
+            self.stack.add_named(scrolled_window, self.database_manager.get_entry_uuid_from_entry_object(self.current_group).urn)
 
         self.switch_stack_page()
 
@@ -332,7 +334,7 @@ class UnlockedDatabase:
             group_page = False
 
         if page_uuid in self.scheduled_page_destroy:
-            stack_page = self.stack.get_child_by_name(page_uuid)
+            stack_page = self.stack.get_child_by_name(page_uuid.urn)
 
             if stack_page is not None:
                 stack_page.destroy()
@@ -340,10 +342,10 @@ class UnlockedDatabase:
             self.scheduled_page_destroy.remove(page_uuid)
             self.show_page_of_new_directory(False, False)
 
-        if self.stack.get_child_by_name(page_uuid) is None:
+        if self.stack.get_child_by_name(page_uuid.urn) is None:
             self.show_page_of_new_directory(False, False)
         else:
-            self.stack.set_visible_child_name(page_uuid)
+            self.stack.set_visible_child_name(page_uuid.urn)
 
         if group_page is True:
             self.set_browser_headerbar()
@@ -351,7 +353,7 @@ class UnlockedDatabase:
             self.entry_page.set_entry_page_headerbar()
 
     def update_current_stack_page(self):
-        stack_page_name = self.database_manager.get_group_uuid_from_group_object(self.current_group)
+        stack_page_name = self.database_manager.get_group_uuid_from_group_object(self.current_group).urn
         stack_page = self.stack.get_child_by_name(stack_page_name)
         stack_page.destroy()
         self.show_page_of_new_directory(False, False)
@@ -373,7 +375,7 @@ class UnlockedDatabase:
             page_uuid = self.database_manager.get_entry_uuid_from_entry_object(self.current_group)
 
         if page_uuid in self.scheduled_page_destroy:
-            stack_page_name = self.stack.get_child_by_name(page_uuid)
+            stack_page_name = self.stack.get_child_by_name(page_uuid.urn)
             if stack_page_name is not None:
                 stack_page_name.destroy()
             self.scheduled_page_destroy.remove(page_uuid)
@@ -626,7 +628,7 @@ class UnlockedDatabase:
             code = ref.group()[5]
 
             try:
-                uuid = self.hex_to_base64(self.reference_to_hex_uuid(ref.group()))
+                uuid = u.UUID(self.reference_to_hex_uuid(ref.group()))
             except(Exception):
                 not_valid = True
 
@@ -647,6 +649,7 @@ class UnlockedDatabase:
                     try:
                         value = self.database_manager.get_entry_password_from_entry_uuid(uuid)
                     except(AttributeError):
+                        print("FAIL")
                         value = ref.group()
                 elif code == "A":
                     try:
@@ -683,7 +686,7 @@ class UnlockedDatabase:
 
     def on_back_button_mobile_clicked(self, button):
         page_uuid = self.database_manager.get_group_uuid_from_group_object(self.current_group)
-        scrolled_page = self.stack.get_child_by_name(page_uuid)
+        scrolled_page = self.stack.get_child_by_name(page_uuid.urn)
 
         if self.database_manager.check_is_group(page_uuid) is True:
             group_page = True
@@ -758,6 +761,7 @@ class UnlockedDatabase:
 
     def lock_database(self):
         self.cancel_timers()
+        self.clipboard.clear()
         self.database_locked = True
         self.unregister_dbus_signal()
         self.stop_save_loop()
@@ -774,6 +778,13 @@ class UnlockedDatabase:
         if self.properties_dialog is not NotImplemented:
             self.properties.close()
 
+        for tmpfile in self.scheduled_tmpfiles_deletion:
+            try:
+                tmpfile.delete()
+            except Exception:
+                self.window.logging_manager.warning("Skipping deletion of tmpfile...")
+
+
         if passwordsafe.config_manager.get_save_automatically() is True:
             save_thread = threading.Thread(target=self.database_manager.save_database)
             save_thread.daemon = False
@@ -790,6 +801,7 @@ class UnlockedDatabase:
         self.cancel_timers()
         self.database_locked = True
         self.stop_save_loop()
+        self.clipboard.clear()
 
         if self.database_settings_dialog is not NotImplemented:
             self.database_settings_dialog.close()
@@ -815,6 +827,26 @@ class UnlockedDatabase:
     # Helper Methods
     #
 
+    def undo_redo_receiver(self, action):
+        if type(self.window.get_focus()) is not Gtk.Entry and type(self.window.get_focus()) is not Gtk.TextView:
+            return
+
+        if type(self.window.get_focus().get_buffer()) is not passwordsafe.history_buffer.HistoryTextBuffer and type(self.window.get_focus().get_buffer()) is not passwordsafe.history_buffer.HistoryEntryBuffer:
+            return
+
+        if "TabBox" not in self.window.get_focus().get_name():
+            return
+
+        buffer = self.window.get_focus().get_buffer()
+        text = ""
+        if action == "undo":
+            text = buffer.logic.do_undo()
+        else:
+            text = buffer.logic.do_redo()
+
+        if text is not None:
+            buffer.set_text(text, len(text))
+
     def clear_clipboard(self):
         clear_clipboard_time = passwordsafe.config_manager.get_clear_clipboard()
         if clear_clipboard_time is not 0:
@@ -888,12 +920,6 @@ class UnlockedDatabase:
     def reference_to_hex_uuid(self, reference_string):
         return reference_string[9:-1].lower()
 
-    def hex_to_base64(self, hex_uuid):
-        return codecs.encode(codecs.decode(hex_uuid.encode(), 'hex'), 'base64').decode().splitlines()[0]
-
-    def base64_to_hex(self, base64_uuid):
-        return codecs.encode(codecs.decode(base64_uuid.encode(), 'base64'), 'hex').decode().upper()
-
     #
     # DBus
     #
@@ -905,4 +931,3 @@ class UnlockedDatabase:
     def unregister_dbus_signal(self):
         app = Gio.Application.get_default
         app().get_dbus_connection().signal_unsubscribe(self.dbus_subscription_id)
-
